﻿using Falcon.SugarApi.ApiClient;
using Microsoft.AspNetCore.Cors.Infrastructure;
using Microsoft.Extensions.DependencyInjection;
using Npgsql.Replication.PgOutput.Messages;
using System;

namespace Falcon.SugarApi.FalconCors
{
    /// <summary>
    /// 服务集合方法扩展
    /// </summary>
    public static class IServiceCollectionExtend
    {
        /// <summary>
        /// 通过cors方式允许所有跨域源通过。Configure中通过app.UseCors("CorsPolicy")运行策略，或在控制器中通过EnableCors应用。
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="policyName">策略名称，默认CorsPolicy</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AllowAny(this IServiceCollection services,string policyName = "CorsPolicy") =>
            services.AddCors(options => options.AddPolicy(policyName,builder => {
                builder
                    .AllowAnyMethod()
                    //.WithMethods("GET","POST","OPTIONS")
                    .AllowAnyHeader()
                    .SetIsOriginAllowed(_ => true) // =AllowAnyOrigin()
                    .AllowCredentials();
            }));

        /// <summary>
        /// 通过cors方式允许所有跨域源通过。Configure中通过app.UseCors("CorsPolicy")运行策略，或在控制器中通过EnableCors应用。
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="policyName">策略名称，默认CorsPolicy</param>
        /// <param name="policyBuilder">策略创建器</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddCorsByAllowAny(
            this IServiceCollection services,
            string policyName,
            Action<CorsPolicyBuilder>? policyBuilder = null) {

            if(policyName.IsNullOrEmpty()) {
                throw new ArgumentNullException(nameof(policyName));
            }
            services.AddCors(options => options.AddPolicy(policyName,builder => {
                builder
                    .AllowAnyMethod()   //.WithMethods("GET","POST","OPTIONS")
                    .AllowAnyHeader()
                    .SetIsOriginAllowed(_ => true) // =AllowAnyOrigin()
                    .AllowCredentials();
                policyBuilder?.Invoke(builder);
            }));
            return services;
        }

        /// <summary>
        /// 添加跨域配置。
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="options">跨域配置选项</param>
        /// <returns>服务集合</returns>
        /// <exception cref="ArgumentNullException"></exception>
        public static IServiceCollection AddCoreWithOptions(this IServiceCollection services,CorsOptions options) {
            if(options == null) {
                throw new ArgumentNullException(nameof(options));
            }
            if(options.PolicyName.IsNullOrEmpty()) {
                throw new ArgumentNullException("options.PolicyName");
            }
            var policy = new CorsOptions();
            services.AddCors(op => op.AddPolicy(options.PolicyName,new CorsPolicyWallpaper(options)));
            return services;
        }
    }
}
