﻿using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using System;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Falcon.SugarApi.TimedTask
{
    /// <summary>
    /// 由定时器驱动的后台任务
    /// </summary>
    public abstract class TimedTask:BackgroundService, IDisposable
    {
        /// <summary>
        /// 日志记录器
        /// </summary>
        protected readonly ILogger? Logger;
        /// <summary>
        /// 心跳计时器
        /// </summary>
        private readonly PeriodicTimer _timer;

        /// <summary>
        /// 运行一次任务
        /// </summary>
        /// <param name="cancellationToken">退出信号</param>
        /// <returns>如果继续执行返回True，否则False</returns>
        public abstract Task<bool> Run(CancellationToken cancellationToken);


        /// <summary>
        /// 执行计划的Cron串
        /// </summary>
        protected abstract string CronSchedule { get; }

        /// <summary>
        /// Timer心跳 毫秒
        /// </summary>
        public virtual int Heartbeat { get; protected set; } = 1000;

        /// <summary>
        /// 获取下次执行任务的计划
        /// </summary>
        protected CronExpression Schedule { get; private set; }

        /// <summary>
        /// 任务正在运行
        /// </summary>
        private bool _isRunning = false;
        /// <summary>
        /// 下次执行时间
        /// </summary>
        public DateTime NextTickTime { get; set; } = DateTime.MinValue;

        /// <summary>
        /// 后台任务开始。返回True任务会继续执行，False则任务退出。
        /// </summary>
        protected virtual bool OnStart(TimedTask t,CancellationToken stoppingToken) {
            return true;
        }
        /// <summary>
        /// 后台任务停止
        /// </summary>
        protected virtual void OnStop(TimedTask t,CancellationToken stoppingToken) { }
        /// <summary>
        /// 完成一次执行。返回True任务会继续执行，False则任务退出。
        /// </summary>
        protected virtual bool OnCompleted(TimedTask t,CancellationToken stoppingToken) {
            return true;
        }
        /// <summary>
        /// 执行中发生未处理异常。返回True任务会继续执行，False则任务退出。
        /// </summary>
        protected virtual bool OnException(TimedTask t,Exception ex,CancellationToken stoppingToken) {
            StringBuilder sb = new();
            sb.AppendLine($"运行后台任务发生未处理的异常，后台任务会继续执行，异常信息可能会重复出现！");
            sb.AppendLine(ex.ToString());
            this.Logger?.LogError(sb.ToString());
            return true;
        }
        /// <summary>
        /// 系统服务
        /// </summary>
        public IServiceProvider Service { get; set; }
        /// <summary>
        /// 资源锁
        /// </summary>
        private static object _lock = new object();

        /// <summary>
        /// 构造Timed后台任务
        /// </summary>
        /// <param name="service">服务提供器</param>
        public TimedTask(IServiceProvider service) {
            this.Service = service;
            this.Logger = service.GetService(typeof(ILogger<>).MakeGenericType(GetType())) as ILogger;
            _timer = new PeriodicTimer(TimeSpan.FromMilliseconds(this.Heartbeat));
            this.Schedule = new CronExpression(this.CronSchedule);
        }

        /// <summary>
        /// 由系统调度执行任务
        /// </summary>
        /// <param name="stoppingToken">退出信号</param>
        /// <returns>无返回</returns>
        protected override async Task ExecuteAsync(CancellationToken stoppingToken) {
            if(!this.OnStart(this,stoppingToken)) {
                return;
            }
            while(await _timer.WaitForNextTickAsync(stoppingToken)) {
                lock(_lock) {
                    if(stoppingToken.IsCancellationRequested) {
                        break;
                    }
                    if(this._isRunning) {
                        continue;
                    }
                    if(DateTime.Now < this.NextTickTime) {
                        continue;
                    }
                    this._isRunning = true;
                }
                try {
                    var goOn = await this.Run(stoppingToken);
                    var comResult = this.OnCompleted(this,stoppingToken);
                    if(!goOn || !comResult) {
                        break;
                    }
                }
                catch(Exception ex) {
                    if(!this.OnException(this,ex,stoppingToken)) {
                        break;
                    }
                }
                SetNextTick();
                this._isRunning = false;
            }
            this.OnStop(this,stoppingToken);
        }

        private void SetNextTick(string? cronSchedule = null) {
            var s = cronSchedule == null ? this.Schedule : new CronExpression(cronSchedule);
            this.NextTickTime = s.GetNextOccurrence(DateTime.Now);
        }

        /// <summary>
        /// 释放对象
        /// </summary>
        public override void Dispose() {
            this._timer?.Dispose();
            GC.SuppressFinalize(this);
            base.Dispose();
        }
    }
}
